@@ -8,6 +8,7 @@ from .base import BaseResource
 
 API_VERSION = '{major}.{minor}.{micro}'.format(**serrano.__version_info__)
 
+
 class Root(BaseResource):
     # Override to allow a POST to not be checked for authorization since
     # this is the only way to authorize.
@@ -46,6 +47,9 @@ class Root(BaseResource):
                 },
                 'exporter': {
                     'href': uri(reverse('serrano:data:exporter')),
+                },
+                'shared_queries': {
+                    'href': uri(reverse('serrano:queries:shared')),
                 }
             }
         }
@@ -58,13 +62,12 @@ class Root(BaseResource):
             if user:
                 login(request, user)
                 token = token_generator.make(user)
-                return {'token': token}
+                data = self.get(request)
+                data['token'] = token
+                return data
         return HttpResponse('Invalid credentials', status=401)
 
 
-
 root_resource = Root()
 
-urlpatterns = patterns('',
-    url(r'^$', root_resource, name='root')
-)
+urlpatterns = patterns('', url(r'^$', root_resource, name='root'))
