@@ -1,14 +1,26 @@
 import functools
+import logging
+from django.http import HttpResponse
 from django.core.urlresolvers import reverse
 from preserialize.serialize import serialize
-from restlib2.params import Parametizer, param_cleaners
+from restlib2.http import codes
+from restlib2.params import Parametizer, StrParam, BoolParam, IntParam
+from avocado.conf import OPTIONAL_DEPS
 from avocado.models import DataField
 from avocado.events import usage
-from ..base import BaseResource
+from ..base import ThrottledResource
 from .. import templates
 
 can_change_field = lambda u: u.has_perm('avocado.change_datafield')
 stats_capable = lambda x: not x.searchable and not x.internal_type == 'auto'
+log = logging.getLogger(__name__)
+
+
+def is_field_orphaned(instance):
+    if instance.model is None or instance.field is None:
+        log.error("Field is an orphan.", extra={'field': instance.pk})
+        return True
+    return False
 
 
 def field_posthook(instance, data, request):
@@ -36,7 +48,8 @@ def field_posthook(instance, data, request):
             'href': uri(reverse('serrano:field-stats', args=[instance.pk])),
         }
         data['_links']['distribution'] = {
-            'href': uri(reverse('serrano:field-distribution', args=[instance.pk])),
+            'href': uri(
+                reverse('serrano:field-distribution', args=[instance.pk])),
         }
 
     return data
@@ -45,41 +58,19 @@ def field_posthook(instance, data, request):
 class FieldParametizer(Parametizer):
     "Supported params and their defaults for Field endpoints."
 
-    sort = None
-    order = 'asc'
-    published = None
-    archived = None
-    brief = False
-    query = ''
-    limit = None
+    sort = StrParam()
+    order = StrParam('asc')
+    published = BoolParam()
+    brief = BoolParam(False)
+    query = StrParam()
+    limit = IntParam()
 
     # Not implemented
-    offset = None
-    page = None
-
-    def clean_published(self, value):
-        return param_cleaners.clean_bool(value)
-
-    def clean_archived(self, value):
-        return param_cleaners.clean_bool(value)
-
-    def clean_brief(self, value):
-        return param_cleaners.clean_bool(value)
-
-    def clean_query(self, value):
-        return param_cleaners.clean_string(value)
-
-    def clean_limit(self, value):
-        return param_cleaners.clean_int(value)
-
-    def clean_offset(self, value):
-        return param_cleaners.clean_int(value)
-
-    def clean_page(self, value):
-        return param_cleaners.clean_int(value)
+    offset = IntParam()
+    page = IntParam()
 
 
-class FieldBase(BaseResource):
+class FieldBase(ThrottledResource):
     model = DataField
 
     parametizer = FieldParametizer
@@ -120,6 +111,13 @@ class FieldResource(FieldBase):
     def get(self, request, pk):
         instance = request.instance
         usage.log('read', instance=instance, request=request)
+
+        # If the field is an orphan then log an error before returning an error
+        if self.checks_for_orphans and is_field_orphaned(instance):
+            return HttpResponse(
+                status=codes.internal_server_error,
+                content="Error occurred when retrieving orphaned field")
+
         return self.prepare(request, instance)
 
 
@@ -141,9 +139,6 @@ class FieldsResource(FieldResource):
             if params['published'] is not None:
                 filters['published'] = params['published']
 
-            if params['archived'] is not None:
-                filters['archived'] = params['archived']
-
             if filters:
                 queryset = queryset.filter(**filters)
         else:
@@ -154,9 +149,9 @@ class FieldsResource(FieldResource):
             usage.log('search', model=self.model, request=request, data={
                 'query': params['query'],
             })
-            results = self.model.objects.search(params['query'],
-                queryset=queryset, max_results=params['limit'],
-                partial=True)
+            results = self.model.objects.search(
+                params['query'], queryset=queryset,
+                max_results=params['limit'], partial=True)
             objects = (x.object for x in results)
         else:
             if params['sort'] == 'name':
@@ -168,4 +163,11 @@ class FieldsResource(FieldResource):
 
             objects = queryset
 
+        if self.checks_for_orphans:
+            pks = []
+            for obj in objects:
+                if not is_field_orphaned(obj):
+                    pks.append(obj.pk)
+            objects = self.model.objects.filter(pk__in=pks)
+
         return self.prepare(request, objects, **params)
