@@ -12,6 +12,10 @@ class FieldStats(FieldBase):
 
         if instance.simple_type == 'number':
             stats = instance.max().min().avg()
+        elif (instance.simple_type == 'date' or
+              instance.simple_type == 'time' or
+              instance.simple_type == 'datetime'):
+            stats = instance.max().min()
         else:
             stats = instance.count(distinct=True)
 
@@ -22,7 +26,8 @@ class FieldStats(FieldBase):
 
         resp['_links'] = {
             'self': {
-                'href': uri(reverse('serrano:field-stats', args=[instance.pk])),
+                'href': uri(
+                    reverse('serrano:field-stats', args=[instance.pk])),
             },
             'parent': {
                 'href': uri(reverse('serrano:field', args=[instance.pk])),
