@@ -1,21 +1,30 @@
 import functools
 import logging
+from datetime import datetime
 from django.http import HttpResponse
 from django.conf.urls import patterns, url
 from django.core.urlresolvers import reverse
+from django.db.models import Q
 from django.views.decorators.cache import never_cache
 from restlib2.http import codes
 from preserialize.serialize import serialize
 from avocado.models import DataQuery
-from avocado.conf import settings
 from avocado.events import usage
+from serrano import utils
+from serrano.decorators import check_auth
 from serrano.forms import QueryForm
-from .base import BaseResource
+from .base import ThrottledResource
+from .history import RevisionsResource, ObjectRevisionsResource, \
+    ObjectRevisionResource
 from . import templates
 
 log = logging.getLogger(__name__)
 
-HISTORY_ENABLED = settings.HISTORY_ENABLED
+DELETE_QUERY_EMAIL_TITLE = "'{0}' has been deleted"
+DELETE_QUERY_EMAIL_BODY = """The query named '{0}' has been deleted. You are
+ being notified because this query was shared with you. This query is no
+ longer available."""
+
 
 def query_posthook(instance, data, request):
     uri = request.build_absolute_uri
@@ -26,7 +35,21 @@ def query_posthook(instance, data, request):
     }
     return data
 
-class QueryBase(BaseResource):
+
+def shared_query_posthook(instance, data, request):
+    data = query_posthook(instance, data, request)
+    data['is_owner'] = instance.user == request.user
+
+    # If this user is not the owner then the shared users are not included
+    # because this sort of data should not be exposed to anyone who isn't the
+    # owner of the original query.
+    if not data['is_owner']:
+        del data['shared_users']
+
+    return data
+
+
+class QueryBase(ThrottledResource):
     cache_max_age = 0
     private_cache = True
 
@@ -70,10 +93,34 @@ class QueryBase(BaseResource):
         log.error('Error creating default query', extra=dict(form.errors))
 
 
-class QueriesResource(QueryBase):
-    "Resource of active (non-archived) queries"
+class SharedQueriesResource(QueryBase):
+    "Resource of shared queries"
+    template = templates.SharedQuery
+
+    def prepare(self, request, instance, template=None):
+        if template is None:
+            template = self.template
+        posthook = functools.partial(shared_query_posthook, request=request)
+        return serialize(instance, posthook=posthook, **template)
+
+    def get_queryset(self, request, **kwargs):
+        if hasattr(request, 'user') and request.user.is_authenticated():
+            f = Q(user=request.user) | Q(shared_users__pk=request.user.pk)
+            return self.model.objects.filter(**kwargs).filter(f).order_by(
+                '-accessed').distinct()
+        else:
+            return self.model.objects.none()
+
+    @check_auth
     def get(self, request):
-        queryset = self.get_queryset(request, archived=False)
+        queryset = self.get_queryset(request)
+        return self.prepare(request, queryset)
+
+
+class QueriesResource(QueryBase):
+    "Resource of queries"
+    def get(self, request):
+        queryset = self.get_queryset(request)
 
         # Only create a default is a session exists
         if request.session.session_key:
@@ -90,23 +137,16 @@ class QueriesResource(QueryBase):
         form = QueryForm(request, request.data)
 
         if form.is_valid():
-            instance = form.save(archive=HISTORY_ENABLED)
+            instance = form.save()
             usage.log('create', instance=instance, request=request)
             response = self.render(request, self.prepare(request, instance),
-                status=codes.created)
+                                   status=codes.created)
         else:
             response = self.render(request, dict(form.errors),
-                status=codes.unprocessable_entity)
+                                   status=codes.unprocessable_entity)
         return response
 
 
-class QueriesHistoryResource(QueryBase):
-    "Resource of archived (non-active) queries"
-    def get(self, request):
-        queryset = self.get_queryset(request, archived=True)
-        return self.prepare(request, queryset)
-
-
 class QueryResource(QueryBase):
     "Resource for accessing a single query"
     def get_object(self, request, pk=None, session=None, **kwargs):
@@ -131,6 +171,8 @@ class QueryResource(QueryBase):
 
     def get(self, request, **kwargs):
         usage.log('read', instance=request.instance, request=request)
+        self.model.objects.filter(pk=request.instance.pk).update(
+            accessed=datetime.now())
         return self.prepare(request, request.instance)
 
     def put(self, request, **kwargs):
@@ -138,32 +180,55 @@ class QueryResource(QueryBase):
         form = QueryForm(request, request.data, instance=instance)
 
         if form.is_valid():
-            instance = form.save(archive=HISTORY_ENABLED)
+            instance = form.save()
             usage.log('update', instance=instance, request=request)
             response = self.render(request, self.prepare(request, instance))
         else:
             response = self.render(request, dict(form.errors),
-                status=codes.unprocessable_entity)
+                                   status=codes.unprocessable_entity)
         return response
 
     def delete(self, request, **kwargs):
         if request.instance.session:
             return HttpResponse(status=codes.bad_request)
+
+        utils.send_mail(
+            request.instance.shared_users.values_list('email', flat=True),
+            DELETE_QUERY_EMAIL_TITLE.format(request.instance.name),
+            DELETE_QUERY_EMAIL_BODY.format(request.instance.name))
+
         request.instance.delete()
-        usage.log('delete', instance=instance, request=request)
+        usage.log('delete', instance=request.instance, request=request)
         return HttpResponse(status=codes.no_content)
 
 
 single_resource = never_cache(QueryResource())
 active_resource = never_cache(QueriesResource())
-history_resource = never_cache(QueriesHistoryResource())
+shared_resource = never_cache(SharedQueriesResource())
+revisions_resource = never_cache(RevisionsResource(
+    object_model=DataQuery, object_model_template=templates.Query,
+    object_model_base_uri='serrano:queries'))
+revisions_for_object_resource = never_cache(ObjectRevisionsResource(
+    object_model=DataQuery, object_model_template=templates.Query,
+    object_model_base_uri='serrano:queries'))
+revision_for_object_resource = never_cache(ObjectRevisionResource(
+    object_model=DataQuery, object_model_template=templates.Query,
+    object_model_base_uri='serrano:queries'))
 
 # Resource endpoints
-urlpatterns = patterns('',
+urlpatterns = patterns(
+    '',
     url(r'^$', active_resource, name='active'),
-    url(r'^history/$', history_resource, name='history'),
+    url(r'^shared/$', shared_resource, name='shared'),
 
     # Endpoints for specific queries
     url(r'^(?P<pk>\d+)/$', single_resource, name='single'),
     url(r'^session/$', single_resource, {'session': True}, name='session'),
+
+    # Revision related endpoints
+    url(r'^revisions/$', revisions_resource, name='revisions'),
+    url(r'^(?P<pk>\d+)/revisions/$', revisions_for_object_resource,
+        name='revisions_for_object'),
+    url(r'^(?P<object_pk>\d+)/revisions/(?P<revision_pk>\d+)/$',
+        revision_for_object_resource, name='revision_for_object'),
 )
