@@ -12,7 +12,7 @@ Field = {
     'fields': [
         ':pk', 'name', 'plural_name', 'description', 'keywords',
         'app_name', 'model_name', 'field_name',
-        'modified', 'published', 'archived', 'operators',
+        'modified', 'published', 'operators',
         'simple_type', 'internal_type', 'data_modified', 'enumerable',
         'searchable', 'unit', 'plural_unit', 'nullable'
     ],
@@ -31,8 +31,8 @@ BriefConcept = {
 Concept = {
     'fields': [
         ':pk', 'name', 'plural_name', 'description', 'keywords',
-        'category_id', 'order', 'modified', 'published', 'archived',
-        'formatter_name', 'queryview', 'sortable'
+        'category_id', 'order', 'modified', 'published',
+        'formatter_name', 'queryable', 'sortable'
     ],
     'aliases': {
         'plural_name': 'get_plural_name',
@@ -61,7 +61,32 @@ View = {
     'allow_missing': True,
 }
 
+User = {
+    'fields': [':pk', 'name', 'username', 'email'],
+    'aliases': {
+        'name': 'get_full_name',
+    }
+}
+
 Query = {
-    'exclude': ['user', 'session_key'],
+    'exclude': ['session_key'],
+    'allow_missing': True,
+    'related': {
+        'user': User,
+        'shared_users': User,
+    }
+}
+
+SharedQuery = {
+    'fields': [':pk', 'accessed', 'name', 'description', 'user',
+               'shared_users', 'context_json', 'view_json'],
+    'related': {
+        'user': User,
+        'shared_users': User,
+    }
+}
+
+Revision = {
+    'exclude': ['user', 'session_key', 'data'],
     'allow_missing': True,
 }
