@@ -1,5 +1,6 @@
 import functools
 import logging
+from datetime import datetime
 from django.http import HttpResponse
 from django.conf.urls import patterns, url
 from django.core.urlresolvers import reverse
@@ -7,15 +8,15 @@ from django.views.decorators.cache import never_cache
 from restlib2.http import codes
 from preserialize.serialize import serialize
 from avocado.models import DataView
-from avocado.conf import settings
 from avocado.events import usage
 from serrano.forms import ViewForm
-from .base import BaseResource
+from .base import ThrottledResource
+from .history import RevisionsResource, ObjectRevisionsResource, \
+    ObjectRevisionResource
 from . import templates
 
 log = logging.getLogger(__name__)
 
-HISTORY_ENABLED = settings.HISTORY_ENABLED
 
 def view_posthook(instance, data, request):
     uri = request.build_absolute_uri
@@ -27,7 +28,7 @@ def view_posthook(instance, data, request):
     return data
 
 
-class ViewBase(BaseResource):
+class ViewBase(ThrottledResource):
     cache_max_age = 0
     private_cache = True
 
@@ -72,9 +73,9 @@ class ViewBase(BaseResource):
 
 
 class ViewsResource(ViewBase):
-    "Resource of active (non-archived) views"
+    "Resource of views"
     def get(self, request):
-        queryset = self.get_queryset(request, archived=False)
+        queryset = self.get_queryset(request)
 
         # Only create a default is a session exists
         if request.session.session_key:
@@ -91,23 +92,16 @@ class ViewsResource(ViewBase):
         form = ViewForm(request, request.data)
 
         if form.is_valid():
-            instance = form.save(archive=HISTORY_ENABLED)
+            instance = form.save()
             usage.log('create', instance=instance, request=request)
             response = self.render(request, self.prepare(request, instance),
-                status=codes.created)
+                                   status=codes.created)
         else:
             response = self.render(request, dict(form.errors),
-                status=codes.unprocessable_entity)
+                                   status=codes.unprocessable_entity)
         return response
 
 
-class ViewsHistoryResource(ViewBase):
-    "Resource of archived (non-active) views"
-    def get(self, request):
-        queryset = self.get_queryset(request, archived=True)
-        return self.prepare(request, queryset)
-
-
 class ViewResource(ViewBase):
     "Resource for accessing a single view"
     def get_object(self, request, pk=None, session=None, **kwargs):
@@ -132,6 +126,8 @@ class ViewResource(ViewBase):
 
     def get(self, request, **kwargs):
         usage.log('read', instance=request.instance, request=request)
+        self.model.objects.filter(pk=request.instance.pk).update(
+            accessed=datetime.now())
         return self.prepare(request, request.instance)
 
     def put(self, request, **kwargs):
@@ -139,32 +135,47 @@ class ViewResource(ViewBase):
         form = ViewForm(request, request.data, instance=instance)
 
         if form.is_valid():
-            instance = form.save(archive=HISTORY_ENABLED)
+            instance = form.save()
             usage.log('update', instance=instance, request=request)
             response = self.render(request, self.prepare(request, instance))
         else:
             response = self.render(request, dict(form.errors),
-                status=codes.unprocessable_entity)
+                                   status=codes.unprocessable_entity)
         return response
 
     def delete(self, request, **kwargs):
         if request.instance.session:
             return HttpResponse(status=codes.bad_request)
         request.instance.delete()
-        usage.log('delete', instance=instance, request=request)
+        usage.log('delete', instance=request.instance, request=request)
         return HttpResponse(status=codes.no_content)
 
 
 single_resource = never_cache(ViewResource())
 active_resource = never_cache(ViewsResource())
-history_resource = never_cache(ViewsHistoryResource())
+revisions_resource = never_cache(RevisionsResource(
+    object_model=DataView, object_model_template=templates.View,
+    object_model_base_uri='serrano:views'))
+revisions_for_object_resource = never_cache(ObjectRevisionsResource(
+    object_model=DataView, object_model_template=templates.View,
+    object_model_base_uri='serrano:views'))
+revision_for_object_resource = never_cache(ObjectRevisionResource(
+    object_model=DataView, object_model_template=templates.View,
+    object_model_base_uri='serrano:views'))
 
 # Resource endpoints
-urlpatterns = patterns('',
+urlpatterns = patterns(
+    '',
     url(r'^$', active_resource, name='active'),
-    url(r'^history/$', history_resource, name='history'),
 
     # Endpoints for specific views
     url(r'^(?P<pk>\d+)/$', single_resource, name='single'),
     url(r'^session/$', single_resource, {'session': True}, name='session'),
+
+    # Revision related endpoints
+    url(r'^revisions/$', revisions_resource, name='revisions'),
+    url(r'^(?P<pk>\d+)/revisions/$', revisions_for_object_resource,
+        name='revisions_for_object'),
+    url(r'^(?P<object_pk>\d+)/revisions/(?P<revision_pk>\d+)/$',
+        revision_for_object_resource, name='revision_for_object'),
 )
